@@ -150,12 +150,68 @@ class UnicodeAttribute(Attribute):
         """
         Returns a unicode string
         """
-        if isinstance(value, six.text_type):
+        if value is None:
+            return None
+        elif isinstance(value, six.text_type):
             return value
         else:
             return six.u(value)
 
 
+class JSONAttribute(Attribute):
+    """
+    A JSON Attribute
+
+    Encodes JSON to unicode internally
+    """
+    def __init__(self, **kwargs):
+        kwargs.setdefault('attr_type', STRING)
+        super(JSONAttribute, self).__init__(**kwargs)
+
+    def serialize(self, value):
+        """
+        Serializes JSON to unicode
+        """
+        if value is None:
+            return None
+        encoded = json.dumps(value)
+        return six.u(encoded)
+
+    def deserialize(self, value):
+        """
+        Deserializes JSON
+        """
+        return json.loads(value)
+
+
+class BooleanAttribute(Attribute):
+    """
+    A class for boolean attributes
+
+    This attribute type uses a number attribute to save space
+    """
+    def __init__(self, **kwargs):
+        kwargs.setdefault('attr_type', NUMBER)
+        super(BooleanAttribute, self).__init__(**kwargs)
+
+    def serialize(self, value):
+        """
+        Encodes True as 1, False as 0
+        """
+        if value is None:
+            return None
+        elif value:
+            return 1
+        else:
+            return 0
+
+    def deserialize(self, value):
+        """
+        Encode
+        """
+        return bool(value)
+
+
 class NumberSetAttribute(SetMixin, Attribute):
     """
     A number set attribute
